@@ -37,25 +37,26 @@ invtransformestimates.exp <- function(x){
 ##' @param omegahat estimate of model parameter omega
 ##' @param Yhat estimate of the latent field  
 ##' @param priors the priors, an object of class 'mcmcPriors', see ?mcmcPriors
-##' @param covmodel an object of class 'covmodel', see ?covmodel
+##' @param cov.model an object of class 'covmodel', see ?covmodel
 ##' @param u vector of distances between points
 ##' @param control list of control parameters, see ?inference.control
 ##' @return estimates of eta, gamma and a proposal variance matrixc for use in the MALA algorithm
 ##' @export
-proposalvariance.exp <- function(X,delta,tm,betahat,omegahat,Yhat,priors,covmodel,u,control){
+proposalvariance.exp <- function(X,delta,tm,betahat,omegahat,Yhat,priors,cov.model,u,control){
      
     n <- length(tm)
     lenbeta <- length(betahat)
     lenomega <- length(omegahat)
-    leneta <- 2
+    leneta <- getleneta(cov.model)
     lenY <- length(Yhat)
     npars <- lenbeta + lenomega + leneta + lenY
     
     sigma <- matrix(0,npars,npars)
     
     # eta
-    logpost <- function(eta,tm,delta,X,beta,omega,Y,priors,covmodel,u){
-        sigmainv <- solve(matrix(getcov(u=u,sigma=exp(eta[1]),phi=exp(eta[2]),model=covmodel$model,pars=covmodel$pars),n,n))
+    logpost <- function(eta,tm,delta,X,beta,omega,Y,priors,cov.model,u){
+        #sigmainv <- solve(matrix(getcov(u=u,sigma=exp(eta[1]),phi=exp(eta[2]),model=cov.model$model,pars=cov.model$pars),n,n))
+        sigmainv <- solve(matrix(EvalCov(cov.model=cov.model,u=u,parameters=eta),n,n))
         cholsigmainv <- t(chol(sigmainv))
         gamma <- cholsigmainv%*%(Y-exp(eta[1])^2/2)                    
         n <- nrow(X)
@@ -66,16 +67,14 @@ proposalvariance.exp <- function(X,delta,tm,betahat,omegahat,Yhat,priors,covmode
         logpost <- sum(delta*(stuff)-expstuff*tm) + priorcontrib # first term, sum(log(diag(cholsigmainv))), is the Jacobian
         return(logpost)
     }
-    ngrid <- 20
-    if(length(priors$etaprior$mean)==1){
-        xseq <- seq(priors$etaprior$mean-1.96*priors$etaprior$sd,priors$etaprior$mean+1.96*priors$etaprior$sd,length.out=ngrid)
-        yseq <- xseq
+
+    npts <- 20
+    if(leneta>=3){
+        npts <- 10
     }
-    else{
-        xseq <- seq(priors$etaprior$mean[1]-1.96*priors$etaprior$sd[1],priors$etaprior$mean[1]+1.96*priors$etaprior$sd[1],length.out=ngrid)
-        yseq <- seq(priors$etaprior$mean[2]-1.96*priors$etaprior$sd[2],priors$etaprior$mean[2]+1.96*priors$etaprior$sd[2],length.out=ngrid)
-    }
-    qa <- quadapprox(logpost,xseq=xseq,yseq=yseq,tm=tm,delta=delta,X=X,beta=betahat,omega=omegahat,Y=Yhat,priors=priors,covmodel=covmodel,u=u)
+    rgs <- getparranges(priors=priors,leneta=leneta)   
+    qa <- QuadApprox(logpost,npts=npts,argRanges=rgs,tm=tm,delta=delta,X=X,beta=betahat,omega=omegahat,Y=Yhat,priors=priors,cov.model=cov.model,u=u)
+    
     matr <- qa$curvature
     etahat <- qa$max
     
@@ -83,7 +82,8 @@ proposalvariance.exp <- function(X,delta,tm,betahat,omegahat,Yhat,priors,covmode
     sigma[(lenbeta+lenomega+1):(lenbeta+lenomega+leneta),(lenbeta+lenomega+1):(lenbeta+lenomega+leneta)] <- matr    
     
     #estimate of gamma
-    Sigma <- matrix(getcov(u=u,sigma=exp(etahat[1]),phi=exp(etahat[2]),model=covmodel$model,pars=covmodel$pars),n,n)
+    #Sigma <- matrix(getcov(u=u,sigma=exp(etahat[1]),phi=exp(etahat[2]),model=cov.model$model,pars=cov.model$pars),n,n)
+    Sigma <- matrix(EvalCov(cov.model=cov.model,u=u,parameters=etahat),n,n)
     covinv <- solve(Sigma)
     cholcovinv <- t(chol(covinv))
     gammahat <- cholcovinv%*%(Yhat-exp(etahat[1])^2/2)  
@@ -128,12 +128,12 @@ proposalvariance.exp <- function(X,delta,tm,betahat,omegahat,Yhat,priors,covmode
 ##' @param omegahat estimate of model parameter omega
 ##' @param Yhat estimate of the latent field  
 ##' @param priors the priors, an object of class 'mcmcPriors', see ?mcmcPriors
-##' @param covmodel an object of class 'covmodel', see ?covmodel
+##' @param cov.model an object of class 'covmodel', see ?covmodel
 ##' @param u vector of distances between points
 ##' @param control list of control parameters, see ?inference.control
 ##' @return estimates of eta, gamma and a proposal variance matrixc for use in the MALA algorithm
 ##' @export
-proposalvariance.exp.gridded <- function(X,delta,tm,betahat,omegahat,Yhat,priors,covmodel,u,control){
+proposalvariance.exp.gridded <- function(X,delta,tm,betahat,omegahat,Yhat,priors,cov.model,u,control){
       
     Ygrid <- gridY(Y=Yhat,control=control)
     
@@ -141,7 +141,7 @@ proposalvariance.exp.gridded <- function(X,delta,tm,betahat,omegahat,Yhat,priors
     n <- length(tm)
     lenbeta <- length(betahat)
     lenomega <- length(omegahat)
-    leneta <- 2
+    leneta <- getleneta(cov.model)
     lenY <- length(Ygrid)
     npars <- lenbeta + lenomega + leneta + lenY
     
@@ -150,12 +150,16 @@ proposalvariance.exp.gridded <- function(X,delta,tm,betahat,omegahat,Yhat,priors
 
     
     # eta
-    logpost <- function(eta,tm,delta,X,beta,omega,Ygrid,priors,covmodel,u,control){
+    logpost <- function(eta,tm,delta,X,beta,omega,Ygrid,priors,cov.model,u,control){
         
-        covbase <- matrix(getcov(u=u,sigma=exp(eta[1]),phi=exp(eta[2]),model=covmodel$model,pars=covmodel$pars),control$Mext,control$Next)
-        rootQeigs <- sqrt(1/Re(fft(covbase)))
+        #covbase <- matrix(getcov(u=u,sigma=exp(eta[1]),phi=exp(eta[2]),model=cov.model$model,pars=cov.model$pars),control$Mext,control$Next)
+        covbase <- matrix(EvalCov(cov.model=cov.model,u=u,parameters=eta),control$Mext,control$Next)
         
-        gamma <- GammafromY(Ygrid,rootQeigs=rootQeigs,mu=-(exp(eta[1]))^2/2)   
+        rootQeigs <- sqrt(1/Re(fft(covbase)))   
+       
+        pars <- sapply(1:length(eta),function(i){cov.model$itrans[[i]](eta[i])})
+        ymean <- -pars[which(cov.model$parnames=="sigma")]^2/2
+        gamma <- GammafromY(Ygrid,rootQeigs=rootQeigs,mu=ymean)   
                           
         n <- nrow(X)
         Xbeta <- X%*%beta        
@@ -165,27 +169,29 @@ proposalvariance.exp.gridded <- function(X,delta,tm,betahat,omegahat,Yhat,priors
         logpost <- sum(delta*(stuff)-expstuff*tm) + priorcontrib
         return(logpost)
     }
-    ngrid <- 20
-    if(length(priors$etaprior$mean)==1){
-        xseq <- seq(priors$etaprior$mean-1.96*priors$etaprior$sd,priors$etaprior$mean+1.96*priors$etaprior$sd,length.out=ngrid)
-        yseq <- xseq
+    
+    npts <- 20
+    if(leneta>=3){
+        npts <- 10
     }
-    else{
-        xseq <- seq(priors$etaprior$mean[1]-1.96*priors$etaprior$sd[1],priors$etaprior$mean[1]+1.96*priors$etaprior$sd[1],length.out=ngrid)
-        yseq <- seq(priors$etaprior$mean[2]-1.96*priors$etaprior$sd[2],priors$etaprior$mean[2]+1.96*priors$etaprior$sd[2],length.out=ngrid)
-    }
-    qa <- quadapprox(logpost,xseq=xseq,yseq=yseq,tm=tm,delta=delta,X=X,beta=betahat,omega=omegahat,Ygrid=Ygrid,priors=priors,covmodel=covmodel,u=u,control=control)
+    rgs <- getparranges(priors=priors,leneta=leneta)   
+    qa <- QuadApprox(logpost,npts=npts,argRanges=rgs,tm=tm,delta=delta,X=X,beta=betahat,omega=omegahat,Ygrid=Ygrid,priors=priors,cov.model=cov.model,u=u,control=control)    
+    
     matr <- qa$curvature
     etahat <- qa$max
+
+    #matt <- mm; nn <- 30; xseq <- yseq <- seq(-0.01,0.01,length.out=nn); grd <- expand.grid(xseq,yseq); zv <- matrix(apply(grd,1,function(x){t(x)%*%matt%*%t(t(x))}),nn,nn); image.plot(xseq,yseq,zv)    
+    
+    #browser()
     
     # entry for eta in propossal covariance
     sigma[(lenbeta+lenomega+1):(lenbeta+lenomega+leneta),(lenbeta+lenomega+1):(lenbeta+lenomega+leneta)] <- matr    
     
     #estimate of gamma
-    covbase <- matrix(getcov(u=u,sigma=exp(etahat[1]),phi=exp(etahat[2]),model=covmodel$model,pars=covmodel$pars),control$Mext,control$Next)
+    covbase <- matrix(EvalCov(cov.model=cov.model,u=u,parameters=etahat),control$Mext,control$Next)
     rootQeigs <- sqrt(1/Re(fft(covbase)))
     invrootQeigs <- 1/rootQeigs
-    gammahat <- GammafromY(Ygrid,rootQeigs=rootQeigs,mu=-(exp(etahat[1]))^2/2)     
+    #gammahat <- GammafromY(Ygrid,rootQeigs=rootQeigs,mu=-(exp(etahat[1]))^2/2)     
         
     deriv <- do.call(priors$derivative,args=list(beta=betahat,omega=omegahat,eta=etahat,priors=priors))
 
@@ -229,7 +235,7 @@ proposalvariance.exp.gridded <- function(X,delta,tm,betahat,omegahat,Yhat,priors
     sigmaret[1:(lenbeta+lenomega+leneta),1:(lenbeta+lenomega+leneta)] <- solve(as.matrix(sigma[1:(lenbeta+lenomega+leneta),1:(lenbeta+lenomega+leneta)]))
     sigmaret[matidx] <- 1/sigma[matidx]   
     
-    return(list(etahat=etahat,gammahat=gammahat,sigma=sigmaret)) 
+    return(list(etahat=etahat,sigma=sigmaret)) 
 }
 
 
